@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,9 @@ static int mode = MSGMODE_NONE;
 static int status = STATUS_CONNECTING;
 static uint16_t mid_sent = 0;
 static bool connected = true;
+static char *username = NULL;
+static char *password = NULL;
+static bool disconnect_sent = false;
 
 void my_connect_callback(void *obj, int result)
 {
@@ -81,7 +84,26 @@ void my_connect_callback(void *obj, int result)
 				break;
 		}
 	}else{
-		fprintf(stderr, "Connect failed\n");
+		switch(result){
+			case 1:
+				fprintf(stderr, "Connection Refused: unacceptable protocol version\n");
+				break;
+			case 2:
+				fprintf(stderr, "Connection Refused: identifier rejected\n");
+				break;
+			case 3:
+				fprintf(stderr, "Connection Refused: broker unavailable\n");
+				break;
+			case 4:
+				fprintf(stderr, "Connection Refused: bad user name or password\n");
+				break;
+			case 5:
+				fprintf(stderr, "Connection Refused: not authorised\n");
+				break;
+			default:
+				fprintf(stderr, "Connection Refused: unknown reason\n");
+				break;
+		}
 	}
 }
 
@@ -94,8 +116,9 @@ void my_publish_callback(void *obj, uint16_t mid)
 {
 	struct mosquitto *mosq = obj;
 
-	if(mode != MSGMODE_STDIN_LINE){
+	if(mode != MSGMODE_STDIN_LINE && disconnect_sent == false){
 		mosquitto_disconnect(mosq);
+		disconnect_sent = true;
 	}
 }
 
@@ -169,6 +192,7 @@ void print_usage(void)
 {
 	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n\n");
 	printf("Usage: mosquitto_pub [-d] [-h host] [-i id] [-p port] [-q qos] [-r] {-f file | -l | -n | -m message} -t topic\n");
+	printf("                     [-u username [--pw password]]\n");
 	printf("                     [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n\n");
 	printf(" -d : enable debug messages.\n");
 	printf(" -f : send the contents of a file as the message.\n");
@@ -182,12 +206,15 @@ void print_usage(void)
 	printf(" -r : message should be retained.\n");
 	printf(" -s : read message from stdin, sending the entire input as a message.\n");
 	printf(" -t : mqtt topic to publish to.\n");
+	printf(" -u : provide a username (requires MQTT 3.1 broker)\n");
+	printf(" --pw : provide a password (requires MQTT 3.1 broker)\n");
 	printf(" --will-payload : payload for the client Will, which is sent by the broker in case of\n");
 	printf("                  unexpected disconnection. If not given and will-topic is set, a zero\n");
 	printf("                  length message will be sent.\n");
 	printf(" --will-qos : QoS level for the client Will.\n");
 	printf(" --will-retain : if given, make the client Will retained.\n");
 	printf(" --will-topic : the topic on which to publish the client Will.\n");
+	printf("\nSee http://mosquitto.org/ for more information.\n\n");
 }
 
 int main(int argc, char *argv[])
@@ -330,6 +357,24 @@ int main(int argc, char *argv[])
 				topic = argv[i+1];
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-u") || !strcmp(argv[i], "--username")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: -u argument given but no username specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				username = argv[i+1];
+			}
+			i++;
+		}else if(!strcmp(argv[i], "--pw")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --pw argument given but no password specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				password = argv[i+1];
+			}
+			i++;
 		}else if(!strcmp(argv[i], "--will-payload")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: --will-payload argument given but no will payload specified.\n\n");
@@ -386,6 +431,9 @@ int main(int argc, char *argv[])
 		print_usage();
 		return 1;
 	}
+	if(password && !username){
+		fprintf(stderr, "Warning: Not using password since username not set.\n");
+	}
 	mosquitto_lib_init();
 	mosq = mosquitto_new(id, NULL);
 	if(!mosq){
@@ -400,6 +448,10 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "Error: Problem setting will.\n");
 		return 1;
 	}
+	if(username && mosquitto_username_pw_set(mosq, username, password)){
+		fprintf(stderr, "Error: Problem setting username and password.\n");
+		return 1;
+	}
 
 	mosquitto_connect_callback_set(mosq, my_connect_callback);
 	mosquitto_disconnect_callback_set(mosq, my_disconnect_callback);
@@ -415,8 +467,9 @@ int main(int argc, char *argv[])
 			if(fgets(buf, 1024, stdin)){
 				buf[strlen(buf)-1] = '\0';
 				mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), (uint8_t *)buf, qos, retain);
-			}else if(feof(stdin)){
+			}else if(feof(stdin) && disconnect_sent == false){
 				mosquitto_disconnect(mosq);
+				disconnect_sent = true;
 			}
 		}
 	}
