@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,8 @@ POSSIBILITY OF SUCH DAMAGE.
 static char **topics = NULL;
 static int topic_count = 0;
 static int topic_qos = 0;
+static char *username = NULL;
+static char *password = NULL;
 int verbose = 0;
 
 void my_message_callback(void *obj, const struct mosquitto_message *message)
@@ -72,7 +74,26 @@ void my_connect_callback(void *obj, int result)
 			mosquitto_subscribe(mosq, NULL, topics[i], topic_qos);
 		}
 	}else{
-		fprintf(stderr, "Connect failed\n");
+		switch(result){
+			case 1:
+				fprintf(stderr, "Connection Refused: unacceptable protocol version\n");
+				break;
+			case 2:
+				fprintf(stderr, "Connection Refused: identifier rejected\n");
+				break;
+			case 3:
+				fprintf(stderr, "Connection Refused: broker unavailable\n");
+				break;
+			case 4:
+				fprintf(stderr, "Connection Refused: bad user name or password\n");
+				break;
+			case 5:
+				fprintf(stderr, "Connection Refused: not authorised\n");
+				break;
+			default:
+				fprintf(stderr, "Connection Refused: unknown reason\n");
+				break;
+		}
 	}
 }
 
@@ -91,6 +112,7 @@ void print_usage(void)
 {
 	printf("mosquitto_sub is a simple mqtt client that will subscribe to a single topic and print all messages it receives.\n\n");
 	printf("Usage: mosquitto_sub [-c] [-i id] [-k keepalive] [-p port] [-q qos] [-v] -t topic ...\n");
+	printf("                     [-u username [--pw password]]\n");
 	printf("                     [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n\n");
 	printf(" -c : disable 'clean session' (store subscription and pending messages when client disconnects).\n");
 	printf(" -d : enable debug messages.\n");
@@ -100,13 +122,16 @@ void print_usage(void)
 	printf(" -p : network port to connect to. Defaults to 1883.\n");
 	printf(" -q : quality of service level to use for the subscription. Defaults to 0.\n");
 	printf(" -t : mqtt topic to subscribe to. May be repeated multiple times.\n");
+	printf(" -u : provide a username (requires MQTT 3.1 broker)\n");
 	printf(" -v : print published messages verbosely.\n");
+	printf(" --pw : provide a password (requires MQTT 3.1 broker)\n");
 	printf(" --will-payload : payload for the client Will, which is sent by the broker in case of\n");
 	printf("                  unexpected disconnection. If not given and will-topic is set, a zero\n");
 	printf("                  length message will be sent.\n");
 	printf(" --will-qos : QoS level for the client Will.\n");
 	printf(" --will-retain : if given, make the client Will retained.\n");
 	printf(" --will-topic : the topic on which to publish the client Will.\n");
+	printf("\nSee http://mosquitto.org/ for more information.\n\n");
 }
 
 int main(int argc, char *argv[])
@@ -205,8 +230,26 @@ int main(int argc, char *argv[])
 				topics[topic_count-1] = argv[i+1];
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-u") || !strcmp(argv[i], "--username")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: -u argument given but no username specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				username = argv[i+1];
+			}
+			i++;
 		}else if(!strcmp(argv[i], "-v") || !strcmp(argv[i], "--verbose")){
 			verbose = 1;
+		}else if(!strcmp(argv[i], "--pw")){
+			if(i==argc-1){
+				fprintf(stderr, "Error: --pw argument given but no password specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				password = argv[i+1];
+			}
+			i++;
 		}else if(!strcmp(argv[i], "--will-payload")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: --will-payload argument given but no will payload specified.\n\n");
@@ -262,6 +305,9 @@ int main(int argc, char *argv[])
 		print_usage();
 		return 1;
 	}
+	if(password && !username){
+		fprintf(stderr, "Warning: Not using password since username not set.\n");
+	}
 	mosquitto_lib_init();
 	mosq = mosquitto_new(id, NULL);
 	if(!mosq){
@@ -276,6 +322,10 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "Error: Problem setting will.\n");
 		return 1;
 	}
+	if(username && mosquitto_username_pw_set(mosq, username, password)){
+		fprintf(stderr, "Error: Problem setting username and password.\n");
+		return 1;
+	}
 	mosquitto_connect_callback_set(mosq, my_connect_callback);
 	mosquitto_message_callback_set(mosq, my_message_callback);
 	if(debug){
