@@ -21,9 +21,10 @@
  * databases to the new mosquitto format. This means a dependency on sqlite. It
  * isn't needed for new installations.
  * Not available on Windows.
+ * This will be removed in version 0.11.
  */
 #ifndef WIN32
-#define WITH_SQLITE_UPGRADE
+//#define WITH_SQLITE_UPGRADE
 #endif
 
 /* Compile with persistent database support. This allows the broker to store
@@ -36,17 +37,12 @@
 #define WITH_PERSISTENCE
 #endif
 
-/* Compile with 32-bit integer database IDs instead of 64-bit integers. May be
- * useful in embedded systems or where be64toh()/htobe64() aren't available.
- * There is the potential for bad things to happen after the IDs wrap around.
- * This is especially likely if there are old retained messages. Note that at a
- * sustained rate of 10,000 messages/s, the database ID would overflow every 5
- * days. It is also worth noting that a broker compiled with 64-bit DB IDs will
- * not load a persistent database file saved from a 32-bit DB ID broker and
- * vice versa.
+/* Compile with bridge support included. This allow the broker to connect to
+ * other brokers and subscribe/publish to topics. You probably want to leave
+ * this included unless you want to save a very small amount of memory size and
+ * CPU time.
  */
-//#define WITH_32BIT_DBID
-
+#define WITH_BRIDGE
 
 /* ============================================================
  * Compatibility defines
@@ -56,4 +52,4 @@
 #ifdef WIN32
 #define snprintf sprintf_s
 #define strcasecmp strcmpi
-#endif
\ No newline at end of file
+#endif
