@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,13 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef _MOSQUITTO_INTERNAL_H_
 #define _MOSQUITTO_INTERNAL_H_
 
+#ifndef CMAKE
+#include <config.h>
+#endif
+
+#ifdef WITH_SSL
+#include <openssl/ssl.h>
+#endif
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -61,12 +68,12 @@ enum mosquitto_client_state {
 
 struct _mosquitto_packet{
 	uint8_t command;
-	uint8_t command_saved;
 	uint8_t have_remaining;
 	uint8_t remaining_count;
 	uint16_t mid;
 	uint32_t remaining_mult;
 	uint32_t remaining_length;
+	uint32_t packet_length;
 	uint32_t to_process;
 	uint32_t pos;
 	uint8_t *payload;
@@ -82,25 +89,47 @@ struct mosquitto_message_all{
 	struct mosquitto_message msg;
 };
 
-struct mosquitto {
-	void *obj;
+#ifdef WITH_SSL
+struct _mosquitto_ssl{
+	SSL_CTX *ssl_ctx;
+	SSL *ssl;
+	BIO *bio;
+	bool want_read;
+	bool want_write;
+};
+#endif
+
+struct _mosquitto_core
+{
 #ifndef WIN32
 	int sock;
 #else
 	SOCKET sock;
 #endif
+	char *address;
 	char *id;
-	int keepalive;
-	unsigned int message_retry;
-	time_t last_retry_check;
+	char *username;
+	char *password;
+	uint16_t keepalive;
+	bool clean_session;
 	enum mosquitto_client_state state;
-	uint16_t last_mid;
-	struct mosquitto_message_all *messages;
-	struct mosquitto_message *will;
-	struct _mosquitto_packet in_packet;
-	struct _mosquitto_packet *out_packet;
 	time_t last_msg_in;
 	time_t last_msg_out;
+	uint16_t last_mid;
+	struct _mosquitto_packet in_packet;
+	struct _mosquitto_packet *out_packet;
+	struct mosquitto_message *will;
+#ifdef WITH_SSL
+	struct _mosquitto_ssl *ssl;
+#endif
+};
+
+struct mosquitto {
+	struct _mosquitto_core core;
+	void *obj;
+	unsigned int message_retry;
+	time_t last_retry_check;
+	struct mosquitto_message_all *messages;
 	int log_priorities;
 	int log_destinations;
 	void (*on_connect)(void *obj, int rc);
