@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -27,9 +27,11 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include <assert.h>
 #include <string.h>
 
 #include <mosquitto.h>
+#include <memory_mosq.h>
 #include <mqtt3_protocol.h>
 #include <net_mosq.h>
 #include <send_mosq.h>
@@ -40,55 +42,74 @@ int _mosquitto_send_connect(struct mosquitto *mosq, uint16_t keepalive, bool cle
 	struct _mosquitto_packet *packet = NULL;
 	int payloadlen;
 	uint8_t will = 0;
+	uint8_t byte;
+	int rc;
 
-	if(!mosq || !mosq->id) return 1;
+	assert(mosq);
+	assert(mosq->core.id);
 
-	packet = calloc(1, sizeof(struct _mosquitto_packet));
-	if(!packet) return 1;
+	packet = _mosquitto_calloc(1, sizeof(struct _mosquitto_packet));
+	if(!packet) return MOSQ_ERR_NOMEM;
 
-	payloadlen = 2+strlen(mosq->id);
-	if(mosq->will){
+	payloadlen = 2+strlen(mosq->core.id);
+	if(mosq->core.will){
 		will = 1;
-		if(mosq->will->topic){
-			payloadlen += 2+strlen(mosq->will->topic) + 2+mosq->will->payloadlen;
-		}else{
-			//FIXME - error
-			return 1;
+		assert(mosq->core.will->topic);
+
+		payloadlen += 2+strlen(mosq->core.will->topic) + 2+mosq->core.will->payloadlen;
+	}
+	if(mosq->core.username){
+		payloadlen += 2+strlen(mosq->core.username);
+		if(mosq->core.password){
+			payloadlen += 2+strlen(mosq->core.password);
 		}
 	}
 
 	packet->command = CONNECT;
 	packet->remaining_length = 12+payloadlen;
-	packet->payload = malloc(sizeof(uint8_t)*(12+payloadlen));
-	if(!packet->payload){
-		free(packet);
-		return 1;
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
+		_mosquitto_free(packet);
+		return rc;
 	}
 
 	/* Variable header */
-	if(_mosquitto_write_string(packet, PROTOCOL_NAME, strlen(PROTOCOL_NAME))) return 1;
-	if(_mosquitto_write_byte(packet, PROTOCOL_VERSION)) return 1;
+	_mosquitto_write_string(packet, PROTOCOL_NAME, strlen(PROTOCOL_NAME));
+	_mosquitto_write_byte(packet, PROTOCOL_VERSION);
+	byte = (clean_session&0x1)<<1;
 	if(will){
-		if(_mosquitto_write_byte(packet, ((mosq->will->retain&0x1)<<5) | ((mosq->will->qos&0x3)<<3) | ((will&0x1)<<2) | ((clean_session&0x1)<<1))) return 1;
-	}else{
-		if(_mosquitto_write_byte(packet, (clean_session&0x1)<<1)) return 1;
+		byte = byte | ((mosq->core.will->retain&0x1)<<5) | ((mosq->core.will->qos&0x3)<<3) | ((will&0x1)<<2);
 	}
-	if(_mosquitto_write_uint16(packet, keepalive)) return 1;
+	if(mosq->core.username){
+		byte = byte | 0x1<<7;
+		if(mosq->core.password){
+			byte = byte | 0x1<<6;
+		}
+	}
+	_mosquitto_write_byte(packet, byte);
+	_mosquitto_write_uint16(packet, keepalive);
 
 	/* Payload */
-	if(_mosquitto_write_string(packet, mosq->id, strlen(mosq->id))) return 1;
+	_mosquitto_write_string(packet, mosq->core.id, strlen(mosq->core.id));
 	if(will){
-		if(_mosquitto_write_string(packet, mosq->will->topic, strlen(mosq->will->topic))) return 1;
-		if(_mosquitto_write_string(packet, (const char *)mosq->will->payload, mosq->will->payloadlen)) return 1;
+		_mosquitto_write_string(packet, mosq->core.will->topic, strlen(mosq->core.will->topic));
+		_mosquitto_write_string(packet, (const char *)mosq->core.will->payload, mosq->core.will->payloadlen);
+	}
+	if(mosq->core.username){
+		_mosquitto_write_string(packet, mosq->core.username, strlen(mosq->core.username));
+		if(mosq->core.password){
+			_mosquitto_write_string(packet, mosq->core.password, strlen(mosq->core.password));
+		}
 	}
 
-	mosq->keepalive = keepalive;
-	if(_mosquitto_packet_queue(mosq, packet)) return 1;
-	return 0;
+	mosq->core.keepalive = keepalive;
+	_mosquitto_packet_queue(&mosq->core, packet);
+	return MOSQ_ERR_SUCCESS;
 }
 
 int _mosquitto_send_disconnect(struct mosquitto *mosq)
 {
+	assert(mosq);
 	return _mosquitto_send_simple_command(mosq, DISCONNECT);
 }
 
@@ -98,33 +119,35 @@ int _mosquitto_send_subscribe(struct mosquitto *mosq, uint16_t *mid, bool dup, c
 	struct _mosquitto_packet *packet = NULL;
 	uint32_t packetlen;
 	uint16_t local_mid;
+	int rc;
 
-	if(!mosq || !topic) return 1;
+	assert(mosq);
+	assert(topic);
 
-	packet = calloc(1, sizeof(struct _mosquitto_packet));
-	if(!packet) return 1;
+	packet = _mosquitto_calloc(1, sizeof(struct _mosquitto_packet));
+	if(!packet) return MOSQ_ERR_NOMEM;
 
 	packetlen = 2 + 2+strlen(topic) + 1;
 
 	packet->command = SUBSCRIBE | (dup<<3) | (1<<1);
 	packet->remaining_length = packetlen;
-	packet->payload = malloc(sizeof(uint8_t)*packetlen);
-	if(!packet->payload){
-		free(packet);
-		return 1;
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
+		_mosquitto_free(packet);
+		return rc;
 	}
 
 	/* Variable header */
-	local_mid = _mosquitto_mid_generate(mosq);
+	local_mid = _mosquitto_mid_generate(&mosq->core);
 	if(mid) *mid = local_mid;
-	if(_mosquitto_write_uint16(packet, local_mid)) return 1;
+	_mosquitto_write_uint16(packet, local_mid);
 
 	/* Payload */
-	if(_mosquitto_write_string(packet, topic, strlen(topic))) return 1;
-	if(_mosquitto_write_byte(packet, topic_qos)) return 1;
+	_mosquitto_write_string(packet, topic, strlen(topic));
+	_mosquitto_write_byte(packet, topic_qos);
 
-	if(_mosquitto_packet_queue(mosq, packet)) return 1;
-	return 0;
+	_mosquitto_packet_queue(&mosq->core, packet);
+	return MOSQ_ERR_SUCCESS;
 }
 
 
@@ -134,31 +157,33 @@ int _mosquitto_send_unsubscribe(struct mosquitto *mosq, uint16_t *mid, bool dup,
 	struct _mosquitto_packet *packet = NULL;
 	uint32_t packetlen;
 	uint16_t local_mid;
+	int rc;
 
-	if(!mosq || !topic) return 1;
+	assert(mosq);
+	assert(topic);
 
-	packet = calloc(1, sizeof(struct _mosquitto_packet));
-	if(!packet) return 1;
+	packet = _mosquitto_calloc(1, sizeof(struct _mosquitto_packet));
+	if(!packet) return MOSQ_ERR_NOMEM;
 
 	packetlen = 2 + 2+strlen(topic);
 
-	packet->command = SUBSCRIBE | (dup<<3) | (1<<1);
+	packet->command = UNSUBSCRIBE | (dup<<3) | (1<<1);
 	packet->remaining_length = packetlen;
-	packet->payload = malloc(sizeof(uint8_t)*packetlen);
-	if(!packet->payload){
-		free(packet);
-		return 1;
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
+		_mosquitto_free(packet);
+		return rc;
 	}
 
 	/* Variable header */
-	local_mid = _mosquitto_mid_generate(mosq);
+	local_mid = _mosquitto_mid_generate(&mosq->core);
 	if(mid) *mid = local_mid;
-	if(_mosquitto_write_uint16(packet, local_mid)) return 1;
+	_mosquitto_write_uint16(packet, local_mid);
 
 	/* Payload */
-	if(_mosquitto_write_string(packet, topic, strlen(topic))) return 1;
+	_mosquitto_write_string(packet, topic, strlen(topic));
 
-	if(_mosquitto_packet_queue(mosq, packet)) return 1;
-	return 0;
+	_mosquitto_packet_queue(&mosq->core, packet);
+	return MOSQ_ERR_SUCCESS;
 }
 
