@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <mqtt3_protocol.h>
 #include <net_mosq.h>
 #include <send_mosq.h>
+#include <util_mosq.h>
 
 static int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t command, uint16_t mid, bool dup);
 
@@ -44,6 +45,7 @@ static int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t comm
 static int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t command, uint16_t mid, bool dup)
 {
 	struct _mosquitto_packet *packet = NULL;
+	int rc;
 
 	assert(mosq);
 	packet = _mosquitto_calloc(1, sizeof(struct _mosquitto_packet));
@@ -54,13 +56,14 @@ static int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t comm
 		packet->command |= 8;
 	}
 	packet->remaining_length = 2;
-	packet->payload = _mosquitto_malloc(sizeof(uint8_t)*2);
-	if(!packet->payload){
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
 		_mosquitto_free(packet);
-		return MOSQ_ERR_NOMEM;
+		return rc;
 	}
-	packet->payload[0] = MOSQ_MSB(mid);
-	packet->payload[1] = MOSQ_LSB(mid);
+
+	packet->payload[packet->pos+0] = MOSQ_MSB(mid);
+	packet->payload[packet->pos+1] = MOSQ_LSB(mid);
 
 	_mosquitto_packet_queue(&mosq->core, packet);
 
@@ -71,6 +74,7 @@ static int _mosquitto_send_command_with_mid(struct mosquitto *mosq, uint8_t comm
 int _mosquitto_send_simple_command(struct mosquitto *mosq, uint8_t command)
 {
 	struct _mosquitto_packet *packet = NULL;
+	int rc;
 
 	assert(mosq);
 	packet = _mosquitto_calloc(1, sizeof(struct _mosquitto_packet));
@@ -79,6 +83,12 @@ int _mosquitto_send_simple_command(struct mosquitto *mosq, uint8_t command)
 	packet->command = command;
 	packet->remaining_length = 0;
 
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
+		_mosquitto_free(packet);
+		return rc;
+	}
+
 	_mosquitto_packet_queue(&mosq->core, packet);
 
 	return MOSQ_ERR_SUCCESS;
@@ -113,13 +123,14 @@ int _mosquitto_send_publish(struct mosquitto *mosq, uint16_t mid, const char *to
 {
 	struct _mosquitto_packet *packet = NULL;
 	int packetlen;
+	int rc;
 
 	assert(mosq);
 	assert(topic);
 
 	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
-	if(mosq) _mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Sending PUBLISH (%d, %d, %d, %d, '%s', ... (%ld bytes))", dup, qos, retain, mid, topic, (long)payloadlen);
+	if(mosq) _mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Sending PUBLISH (d%d, q%d, r%d, m%d, '%s', ... (%ld bytes))", dup, qos, retain, mid, topic, (long)payloadlen);
 
 	packetlen = 2+strlen(topic) + payloadlen;
 	if(qos > 0) packetlen += 2; /* For message id */
@@ -131,13 +142,11 @@ int _mosquitto_send_publish(struct mosquitto *mosq, uint16_t mid, const char *to
 
 	packet->mid = mid;
 	packet->command = PUBLISH | ((dup&0x1)<<3) | (qos<<1) | retain;
-	packet->command_saved = PUBLISH | (qos<<1);
 	packet->remaining_length = packetlen;
-	packet->payload = _mosquitto_malloc(sizeof(uint8_t)*packetlen);
-	if(!packet->payload){
-		_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "PUBLISH failed allocating payload memory.");
+	rc = _mosquitto_packet_alloc(packet);
+	if(rc){
 		_mosquitto_free(packet);
-		return MOSQ_ERR_NOMEM;
+		return rc;
 	}
 	/* Variable header (topic string) */
 	_mosquitto_write_string(packet, topic, strlen(topic));
