@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <ws2tcpip.h>
 #endif
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 #include <memory_mosq.h>
 
@@ -54,7 +57,7 @@ mqtt3_context *mqtt3_context_init(int sock)
 	context->core.last_msg_in = time(NULL);
 	context->core.last_msg_out = time(NULL);
 	context->core.keepalive = 60; /* Default to 60s */
-	context->clean_session = true;
+	context->core.clean_session = true;
 	context->core.id = NULL;
 	context->core.last_mid = 0;
 	context->core.will = NULL;
@@ -66,26 +69,29 @@ mqtt3_context *mqtt3_context_init(int sock)
 	context->core.out_packet = NULL;
 
 	addrlen = sizeof(addr);
-	context->address = NULL;
+	context->core.address = NULL;
 	if(!getpeername(sock, (struct sockaddr *)&addr, &addrlen)){
 		if(addr.ss_family == AF_INET){
 			if(inet_ntop(AF_INET, &((struct sockaddr_in *)&addr)->sin_addr.s_addr, address, 1024)){
-				context->address = _mosquitto_strdup(address);
+				context->core.address = _mosquitto_strdup(address);
 			}
 		}else if(addr.ss_family == AF_INET6){
 			if(inet_ntop(AF_INET6, &((struct sockaddr_in6 *)&addr)->sin6_addr.s6_addr, address, 1024)){
-				context->address = _mosquitto_strdup(address);
+				context->core.address = _mosquitto_strdup(address);
 			}
 		}
 	}
-	if(!context->address && sock != -1){
+	if(!context->core.address && sock != -1){
 		/* getpeername and inet_ntop failed and not a bridge */
 		_mosquitto_free(context);
 		return NULL;
 	}
 	context->bridge = NULL;
 	context->msgs = NULL;
-	
+#ifdef WITH_SSL
+	context->core.ssl = NULL;
+#endif
+
 	return context;
 }
 
@@ -102,15 +108,15 @@ void mqtt3_context_cleanup(mosquitto_db *db, mqtt3_context *context, bool do_fre
 	if(!context) return;
 
 	if(context->core.sock != -1){
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 	}
-	if(context->clean_session && !context->duplicate && db){
+	if(context->core.clean_session && !context->duplicate && db){
 		mqtt3_subs_clean_session(context, &db->subs);
 		mqtt3_db_messages_delete(context);
 	}
-	if(context->address){
-		_mosquitto_free(context->address);
-		context->address = NULL;
+	if(context->core.address){
+		_mosquitto_free(context->core.address);
+		context->core.address = NULL;
 	}
 	if(context->core.id){
 		_mosquitto_free(context->core.id);
@@ -128,7 +134,7 @@ void mqtt3_context_cleanup(mosquitto_db *db, mqtt3_context *context, bool do_fre
 		if(context->core.will->payload) _mosquitto_free(context->core.will->payload);
 		_mosquitto_free(context->core.will);
 	}
-	if(do_free || context->clean_session){
+	if(do_free || context->core.clean_session){
 		msg = context->msgs;
 		while(msg){
 			next = msg->next;
