@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -65,9 +65,13 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <string.h>
 #include <sys/stat.h>
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 #include <memory_mosq.h>
+#include <mosquitto.h>
 #include <util_mosq.h>
 
 static int max_inflight = 20;
@@ -80,7 +84,7 @@ int mqtt3_db_open(mqtt3_config *config, mosquitto_db *db)
 	int rc = 0;
 	struct _mosquitto_subhier *child;
 
-	if(!config) return 1;
+	if(!config || !db) return MOSQ_ERR_INVAL;
 
 	db->last_db_id = 0;
 
@@ -94,44 +98,65 @@ int mqtt3_db_open(mqtt3_config *config, mosquitto_db *db)
 	db->subs.topic = "";
 
 	child = _mosquitto_malloc(sizeof(struct _mosquitto_subhier));
-	if(!child) return MOSQ_ERR_NOMEM;
+	if(!child){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->next = NULL;
 	child->topic = _mosquitto_strdup("");
-	if(!child->topic) return MOSQ_ERR_NOMEM;
+	if(!child->topic){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->subs = NULL;
 	child->children = NULL;
 	child->retained = NULL;
 	db->subs.children = child;
 
 	child = _mosquitto_malloc(sizeof(struct _mosquitto_subhier));
-	if(!child) return MOSQ_ERR_NOMEM;
+	if(!child){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->next = NULL;
 	child->topic = _mosquitto_strdup("/");
-	if(!child->topic) return MOSQ_ERR_NOMEM;
+	if(!child->topic){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->subs = NULL;
 	child->children = NULL;
 	child->retained = NULL;
 	db->subs.children->next = child;
 
 	child = _mosquitto_malloc(sizeof(struct _mosquitto_subhier));
-	if(!child) return MOSQ_ERR_NOMEM;
+	if(!child){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->next = NULL;
 	child->topic = _mosquitto_strdup("$SYS");
-	if(!child->topic) return MOSQ_ERR_NOMEM;
+	if(!child->topic){
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	child->subs = NULL;
 	child->children = NULL;
 	child->retained = NULL;
 	db->subs.children->next->next = child;
 
+	db->unpwd = NULL;
+
 #ifdef WITH_PERSISTENCE
 	if(config->persistence){
 		if(config->persistence_location && strlen(config->persistence_location)){
 			db->filepath = _mosquitto_malloc(strlen(config->persistence_location) + strlen(config->persistence_file) + 1);
-			if(!db->filepath) return 1;
+			if(!db->filepath) return MOSQ_ERR_NOMEM;
 			sprintf(db->filepath, "%s%s", config->persistence_location, config->persistence_file);
 			if(mqtt3_db_restore(db)) return 1;
 		}else{
 			db->filepath = _mosquitto_strdup(config->persistence_file);
+			if(!db->filepath) return MOSQ_ERR_NOMEM;
 			if(mqtt3_db_restore(db)) return 1;
 		}
 	}
@@ -182,8 +207,8 @@ int mqtt3_db_close(mosquitto_db *db)
  */
 int mqtt3_db_client_will_queue(mosquitto_db *db, mqtt3_context *context)
 {
-	if(!context || !context->core.id) return 1;
-	if(!context->core.will) return 0;
+	if(!context || !context->core.id) return MOSQ_ERR_INVAL;
+	if(!context->core.will) return MOSQ_ERR_SUCCESS;
 
 	return mqtt3_db_messages_easy_queue(db, context, context->core.will->topic, context->core.will->qos, context->core.will->payloadlen, context->core.will->payload, context->core.will->retain);
 }
@@ -197,14 +222,14 @@ int mqtt3_db_client_count(mosquitto_db *db, int *count)
 {
 	int i;
 
-	if(!db || !count) return 1;
+	if(!db || !count) return MOSQ_ERR_INVAL;
 
 	*count = 0;
 	for(i=0; i<db->context_count; i++){
 		if(db->contexts[i]) (*count)++;
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 /* Internal function.
@@ -217,7 +242,7 @@ static int _mqtt3_db_cleanup(mosquitto_db *db)
 {
 	int rc = 0;
 
-	if(!db) return 1;
+	if(!db) return MOSQ_ERR_INVAL;
 
 // FIXME - reimplement for new db
 #if 0
@@ -238,7 +263,7 @@ int mqtt3_db_message_delete(mqtt3_context *context, uint16_t mid, enum mosquitto
 	int msg_index = 0;
 	bool deleted = false;
 
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 
 	tail = context->msgs;
 	while(tail){
@@ -284,11 +309,11 @@ int mqtt3_db_message_delete(mqtt3_context *context, uint16_t mid, enum mosquitto
 			tail = tail->next;
 		}
 		if(msg_index > max_inflight && deleted){
-			return 0;
+			return MOSQ_ERR_SUCCESS;
 		}
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_db_message_insert(mqtt3_context *context, uint16_t mid, enum mosquitto_msg_direction dir, int qos, bool retain, struct mosquitto_msg_store *stored)
@@ -299,7 +324,7 @@ int mqtt3_db_message_insert(mqtt3_context *context, uint16_t mid, enum mosquitto
 	int rc = 0;
 
 	assert(stored);
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 
 	if(context->msgs){
 		tail = context->msgs;
@@ -343,7 +368,7 @@ int mqtt3_db_message_insert(mqtt3_context *context, uint16_t mid, enum mosquitto
 	assert(state != ms_invalid);
 
 	msg = _mosquitto_malloc(sizeof(mosquitto_client_msg));
-	if(!msg) return 1;
+	if(!msg) return MOSQ_ERR_NOMEM;
 	msg->next = NULL;
 	msg->store = stored;
 	msg->store->ref_count++;
@@ -372,7 +397,7 @@ int mqtt3_db_message_update(mqtt3_context *context, uint16_t mid, enum mosquitto
 		if(tail->mid == mid && tail->direction == dir){
 			tail->state = state;
 			tail->timestamp = time(NULL);
-			return 0;
+			return MOSQ_ERR_SUCCESS;
 		}
 		tail = tail->next;
 	}
@@ -383,7 +408,7 @@ int mqtt3_db_messages_delete(mqtt3_context *context)
 {
 	mosquitto_client_msg *tail, *next;
 
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 
 	tail = context->msgs;
 	while(tail){
@@ -395,7 +420,7 @@ int mqtt3_db_messages_delete(mqtt3_context *context)
 	}
 	context->msgs = NULL;
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_db_messages_easy_queue(mosquitto_db *db, mqtt3_context *context, const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain)
@@ -405,7 +430,7 @@ int mqtt3_db_messages_easy_queue(mosquitto_db *db, mqtt3_context *context, const
 
 	assert(db);
 
-	if(!topic) return 1;
+	if(!topic) return MOSQ_ERR_INVAL;
 
 	if(context){
 		source_id = context->core.id;
@@ -425,9 +450,9 @@ int mqtt3_db_messages_queue(mosquitto_db *db, const char *source_id, const char
 	assert(stored);
 
 	/* Find all clients that subscribe to topic and put messages into the db for them. */
-	if(!source_id || !topic) return 1;
+	if(!source_id || !topic) return MOSQ_ERR_INVAL;
 
-	mqtt3_sub_search(&db->subs, source_id, topic, qos, retain, stored);
+	mqtt3_sub_search(db, &db->subs, source_id, topic, qos, retain, stored);
 	return rc;
 }
 
@@ -438,10 +463,10 @@ int mqtt3_db_message_store(mosquitto_db *db, const char *source, uint16_t source
 	assert(db);
 	assert(stored);
 
-	if(!topic) return 1;
+	if(!topic) return MOSQ_ERR_INVAL;
 
 	temp = _mosquitto_malloc(sizeof(struct mosquitto_msg_store));
-	if(!temp) return 1;
+	if(!temp) return MOSQ_ERR_NOMEM;
 
 	temp->next = db->msg_store;
 	temp->ref_count = 0;
@@ -450,16 +475,32 @@ int mqtt3_db_message_store(mosquitto_db *db, const char *source, uint16_t source
 	}else{
 		temp->source_id = _mosquitto_strdup("");
 	}
+	if(!temp->source_id){
+		_mosquitto_free(temp);
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	temp->source_mid = source_mid;
 	temp->msg.qos = qos;
 	temp->msg.retain = retain;
 	temp->msg.topic = _mosquitto_strdup(topic);
+	if(!temp->msg.topic){
+		_mosquitto_free(temp);
+		_mosquitto_free(temp->source_id);
+		mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
 	temp->msg.payloadlen = payloadlen;
 	if(payloadlen){
 		temp->msg.payload = _mosquitto_malloc(sizeof(uint8_t)*payloadlen);
-		if(temp->msg.payload){
-			memcpy(temp->msg.payload, payload, sizeof(uint8_t)*payloadlen);
+		if(!temp->msg.payload){
+			if(temp->source_id) _mosquitto_free(temp->source_id);
+			if(temp->msg.topic) _mosquitto_free(temp->msg.topic);
+			if(temp->msg.payload) _mosquitto_free(temp->msg.payload);
+			_mosquitto_free(temp);
+			return MOSQ_ERR_NOMEM;
 		}
+		memcpy(temp->msg.payload, payload, sizeof(uint8_t)*payloadlen);
 	}else{
 		temp->msg.payload = NULL;
 	}
@@ -481,7 +522,7 @@ int mqtt3_db_message_store(mosquitto_db *db, const char *source, uint16_t source
 		temp->db_id = store_id;
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_db_message_store_find(mosquitto_db *db, const char *source, uint16_t mid, struct mosquitto_msg_store **stored)
@@ -493,7 +534,7 @@ int mqtt3_db_message_store_find(mosquitto_db *db, const char *source, uint16_t m
 	while(tail){
 		if(tail->source_mid == mid && !strcmp(tail->source_id, source)){
 			*stored = tail;
-			return 0;
+			return MOSQ_ERR_SUCCESS;
 		}
 		tail = tail->next;
 	}
@@ -554,7 +595,7 @@ int mqtt3_db_message_release(mosquitto_db *db, mqtt3_context *context, uint16_t
 	char *topic;
 	char *source_id;
 
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 
 	tail = context->msgs;
 	while(tail){
@@ -572,7 +613,7 @@ int mqtt3_db_message_release(mosquitto_db *db, mqtt3_context *context, uint16_t
 					context->msgs = tail->next;
 				}
 				_mosquitto_free(tail);
-				return 0;
+				return MOSQ_ERR_SUCCESS;
 			}else{
 				return 1;
 			}
@@ -585,6 +626,7 @@ int mqtt3_db_message_release(mosquitto_db *db, mqtt3_context *context, uint16_t
 
 int mqtt3_db_message_write(mqtt3_context *context)
 {
+	int rc;
 	mosquitto_client_msg *tail, *last = NULL;
 	uint16_t mid;
 	int retries;
@@ -594,7 +636,10 @@ int mqtt3_db_message_write(mqtt3_context *context)
 	uint32_t payloadlen;
 	const uint8_t *payload;
 
-	if(!context || !context->core.id || context->core.sock == -1) return 1;
+	if(!context || context->core.sock == -1
+			|| (context->core.state == mosq_cs_connected && !context->core.id)){
+		return MOSQ_ERR_INVAL;
+	}
 
 	tail = context->msgs;
 	while(tail){
@@ -609,7 +654,8 @@ int mqtt3_db_message_write(mqtt3_context *context)
 
 			switch(tail->state){
 				case ms_publish:
-					if(!mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload)){
+					rc = mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload);
+					if(!rc){
 						if(last){
 							last->next = tail->next;
 							tail->store->ref_count--;
@@ -621,44 +667,61 @@ int mqtt3_db_message_write(mqtt3_context *context)
 							_mosquitto_free(tail);
 							tail = context->msgs;
 						}
+					}else{
+						return rc;
 					}
 					break;
 
 				case ms_publish_puback:
-					if(!mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload)){
+					rc = mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload);
+					if(!rc){
 						tail->state = ms_wait_puback;
+					}else{
+						return rc;
 					}
 					last = tail;
 					tail = tail->next;
 					break;
 
 				case ms_publish_pubrec:
-					if(!mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload)){
+					rc = mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload);
+					if(!rc){
 						tail->state = ms_wait_pubrec;
+					}else{
+						return rc;
 					}
 					last = tail;
 					tail = tail->next;
 					break;
 				
 				case ms_resend_pubrec:
-					if(!mqtt3_raw_pubrec(context, mid)){
+					rc = mqtt3_raw_pubrec(context, mid);
+					if(!rc){
 						tail->state = ms_wait_pubrel;
+					}else{
+						return rc;
 					}
 					last = tail;
 					tail = tail->next;
 					break;
 
 				case ms_resend_pubrel:
-					if(!mqtt3_raw_pubrel(context, mid, true)){
+					rc = mqtt3_raw_pubrel(context, mid, true);
+					if(!rc){
 						tail->state = ms_wait_pubcomp;
+					}else{
+						return rc;
 					}
 					last = tail;
 					tail = tail->next;
 					break;
 
 				case ms_resend_pubcomp:
-					if(!mqtt3_raw_pubcomp(context, mid)){
+					rc = mqtt3_raw_pubcomp(context, mid);
+					if(!rc){
 						tail->state = ms_wait_pubrel;
+					}else{
+						return rc;
 					}
 					last = tail;
 					tail = tail->next;
@@ -675,7 +738,7 @@ int mqtt3_db_message_write(mqtt3_context *context)
 		}
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 void mqtt3_db_store_clean(mosquitto_db *db)
