@@ -29,11 +29,22 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
+#ifndef WIN32
 #include <syslog.h>
+#endif
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 
+extern mosquitto_db int_db;
+
+#ifdef WIN32
+HANDLE syslog_h;
+#endif
+
 /* Options for logging should be:
  *
  * A combination of:
@@ -47,7 +58,7 @@ POSSIBILITY OF SUCH DAMAGE.
  * Logging pid.
  */
 static int log_destinations = MQTT3_LOG_STDERR;
-static int log_priorities = MQTT3_LOG_ERR | MQTT3_LOG_WARNING | MQTT3_LOG_NOTICE | MQTT3_LOG_INFO;
+static int log_priorities = MOSQ_LOG_ERR | MOSQ_LOG_WARNING | MOSQ_LOG_NOTICE | MOSQ_LOG_INFO;
 
 int mqtt3_log_init(int priorities, int destinations)
 {
@@ -57,7 +68,11 @@ int mqtt3_log_init(int priorities, int destinations)
 	log_destinations = destinations;
 
 	if(log_destinations & MQTT3_LOG_SYSLOG){
+#ifndef WIN32
 		openlog("mosquitto", LOG_PID, LOG_DAEMON);
+#else
+		syslog_h = OpenEventLog(NULL, "mosquitto");
+#endif
 	}
 
 	return rc;
@@ -66,45 +81,76 @@ int mqtt3_log_init(int priorities, int destinations)
 int mqtt3_log_close(void)
 {
 	if(log_destinations & MQTT3_LOG_SYSLOG){
+#ifndef WIN32
 		closelog();
+#else
+		CloseEventLog(syslog_h);
+#endif
 	}
 	/* FIXME - do something for all destinations! */
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_log_printf(int priority, const char *fmt, ...)
 {
 	va_list va;
 	char s[500];
+#ifdef WIN32
+	char *sp;
+#endif
 	const char *topic;
 	int syslog_priority;
 
 	if((log_priorities & priority) && log_destinations != MQTT3_LOG_NONE){
 		switch(priority){
-			case MQTT3_LOG_DEBUG:
+			case MOSQ_LOG_DEBUG:
 				topic = "$SYS/broker/log/D";
+#ifndef WIN32
 				syslog_priority = LOG_DEBUG;
+#else
+				syslog_priority = EVENTLOG_INFORMATION_TYPE;
+#endif
 				break;
-			case MQTT3_LOG_ERR:
+			case MOSQ_LOG_ERR:
 				topic = "$SYS/broker/log/E";
+#ifndef WIN32
 				syslog_priority = LOG_ERR;
+#else
+				syslog_priority = EVENTLOG_ERROR_TYPE;
+#endif
 				break;
-			case MQTT3_LOG_WARNING:
+			case MOSQ_LOG_WARNING:
 				topic = "$SYS/broker/log/W";
+#ifndef WIN32
 				syslog_priority = LOG_WARNING;
+#else
+				syslog_priority = EVENTLOG_WARNING_TYPE;
+#endif
 				break;
-			case MQTT3_LOG_NOTICE:
+			case MOSQ_LOG_NOTICE:
 				topic = "$SYS/broker/log/N";
+#ifndef WIN32
 				syslog_priority = LOG_NOTICE;
+#else
+				syslog_priority = EVENTLOG_INFORMATION_TYPE;
+#endif
 				break;
-			case MQTT3_LOG_INFO:
+			case MOSQ_LOG_INFO:
 				topic = "$SYS/broker/log/I";
+#ifndef WIN32
 				syslog_priority = LOG_INFO;
+#else
+				syslog_priority = EVENTLOG_INFORMATION_TYPE;
+#endif
 				break;
 			default:
 				topic = "$SYS/broker/log/E";
+#ifndef WIN32
 				syslog_priority = LOG_ERR;
+#else
+				syslog_priority = EVENTLOG_ERROR_TYPE;
+#endif
 		}
 		va_start(va, fmt);
 		vsnprintf(s, 500, fmt, va);
@@ -119,13 +165,18 @@ int mqtt3_log_printf(int priority, const char *fmt, ...)
 			fflush(stderr);
 		}
 		if(log_destinations & MQTT3_LOG_SYSLOG){
+#ifndef WIN32
 			syslog(syslog_priority, "%s", s);
+#else
+			sp = (char *)s;
+			ReportEvent(syslog_h, syslog_priority, 0, 0, NULL, 1, 0, &sp, NULL);
+#endif
 		}
-		if(log_destinations & MQTT3_LOG_TOPIC && priority != MQTT3_LOG_DEBUG){
-			mqtt3_db_messages_easy_queue("", topic, 2, strlen(s), (uint8_t *)s, 0);
+		if(log_destinations & MQTT3_LOG_TOPIC && priority != MOSQ_LOG_DEBUG){
+			mqtt3_db_messages_easy_queue(&int_db, NULL, topic, 2, strlen(s), (uint8_t *)s, 0);
 		}
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
