@@ -29,7 +29,10 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <string.h>
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 #include <mqtt3_protocol.h>
 #include <memory_mosq.h>
@@ -37,7 +40,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 int mqtt3_packet_handle(mosquitto_db *db, mqtt3_context *context)
 {
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 
 	switch((context->core.in_packet.command)&0xF0){
 		case PINGREQ:
@@ -54,7 +57,6 @@ int mqtt3_packet_handle(mosquitto_db *db, mqtt3_context *context)
 			return mqtt3_handle_pubrec(context);
 		case PUBREL:
 			return mqtt3_handle_pubrel(db, context);
-#ifdef WITH_BROKER
 		case CONNECT:
 			return mqtt3_handle_connect(db, context);
 		case DISCONNECT:
@@ -63,7 +65,6 @@ int mqtt3_packet_handle(mosquitto_db *db, mqtt3_context *context)
 			return mqtt3_handle_subscribe(db, context);
 		case UNSUBSCRIBE:
 			return mqtt3_handle_unsubscribe(db, context);
-#endif
 #ifdef WITH_BRIDGE
 		case CONNACK:
 			return mqtt3_handle_connack(context);
@@ -83,7 +84,7 @@ int mqtt3_handle_puback(mqtt3_context *context)
 	uint16_t mid;
 
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_PROTOCOL;
@@ -100,7 +101,7 @@ int mqtt3_handle_puback(mqtt3_context *context)
 int mqtt3_handle_pingreq(mqtt3_context *context)
 {
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 0){
 		return MOSQ_ERR_PROTOCOL;
@@ -112,7 +113,7 @@ int mqtt3_handle_pingreq(mqtt3_context *context)
 int mqtt3_handle_pingresp(mqtt3_context *context)
 {
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 0){
 		return MOSQ_ERR_PROTOCOL;
@@ -126,7 +127,7 @@ int mqtt3_handle_pubcomp(mqtt3_context *context)
 	uint16_t mid;
 
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_PROTOCOL;
@@ -180,6 +181,18 @@ int mqtt3_handle_publish(mosquitto_db *db, mqtt3_context *context)
 		}
 	}
 
+	/* Check for topic access */
+	rc = mqtt3_acl_check(db, context, topic, MOSQ_ACL_WRITE);
+	if(rc == MOSQ_ERR_ACL_DENIED){
+		_mosquitto_free(topic);
+		if(payload) _mosquitto_free(payload);
+		return MOSQ_ERR_SUCCESS;
+	}else if(rc != MOSQ_ERR_SUCCESS){
+		_mosquitto_free(topic);
+		if(payload) _mosquitto_free(payload);
+		return rc;
+	}
+
 	if(qos > 0){
 		mqtt3_db_message_store_find(db, context->core.id, mid, &stored);
 	}
@@ -203,7 +216,7 @@ int mqtt3_handle_publish(mosquitto_db *db, mqtt3_context *context)
 			break;
 		case 2:
 			if(!dup){
-				res = mqtt3_db_message_insert(context, mid, mosq_md_in, qos, false, stored);
+				res = mqtt3_db_message_insert(context, mid, mosq_md_in, qos, retain, stored);
 			}else{
 				res = 0;
 			}
@@ -225,7 +238,7 @@ int mqtt3_handle_pubrec(mqtt3_context *context)
 	uint16_t mid;
 
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_NOMEM;
@@ -244,7 +257,7 @@ int mqtt3_handle_pubrel(mosquitto_db *db, mqtt3_context *context)
 	uint16_t mid;
 
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 2){
 		return MOSQ_ERR_NOMEM;
