@@ -27,7 +27,10 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 
 void (*client_connack_callback)(int) = NULL;
@@ -38,12 +41,15 @@ int mqtt3_handle_connack(mqtt3_context *context)
 	uint8_t rc;
 	int i;
 
-	if(!context || context->in_packet.remaining_length != 2){
-		return 1;
+	if(!context){
+		return MOSQ_ERR_INVAL;
 	}
-	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received CONNACK");
-	if(_mosquitto_read_byte(&context->in_packet, &byte)) return 1; // Reserved byte, not used
-	if(_mosquitto_read_byte(&context->in_packet, &rc)) return 1;
+	if(context->core.in_packet.remaining_length != 2){
+		return MOSQ_ERR_PROTOCOL;
+	}
+	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received CONNACK");
+	if(_mosquitto_read_byte(&context->core.in_packet, &byte)) return 1; // Reserved byte, not used
+	if(_mosquitto_read_byte(&context->core.in_packet, &rc)) return 1;
 	if(client_connack_callback){
 		client_connack_callback(rc);
 	}
@@ -58,16 +64,16 @@ int mqtt3_handle_connack(mqtt3_context *context)
 					}
 				}
 			}
-			context->connected = true;
-			return 0;
+			context->core.state = mosq_cs_connected;
+			return MOSQ_ERR_SUCCESS;
 		case 1:
-			mqtt3_log_printf(MQTT3_LOG_ERR, "Connection Refused: unacceptable protocol version");
+			mqtt3_log_printf(MOSQ_LOG_ERR, "Connection Refused: unacceptable protocol version");
 			return 1;
 		case 2:
-			mqtt3_log_printf(MQTT3_LOG_ERR, "Connection Refused: identifier rejected");
+			mqtt3_log_printf(MOSQ_LOG_ERR, "Connection Refused: identifier rejected");
 			return 1;
 		case 3:
-			mqtt3_log_printf(MQTT3_LOG_ERR, "Connection Refused: broker unavailable");
+			mqtt3_log_printf(MOSQ_LOG_ERR, "Connection Refused: broker unavailable");
 			return 1;
 	}
 	return 1;
@@ -78,26 +84,29 @@ int mqtt3_handle_suback(mqtt3_context *context)
 	uint16_t mid;
 	uint8_t granted_qos;
 
-	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received SUBACK");
-	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
+	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received SUBACK");
+	if(_mosquitto_read_uint16(&context->core.in_packet, &mid)) return 1;
 
-	while(context->in_packet.pos < context->in_packet.remaining_length){
+	while(context->core.in_packet.pos < context->core.in_packet.remaining_length){
 		/* FIXME - Need to do something with this */
-		if(_mosquitto_read_byte(&context->in_packet, &granted_qos)) return 1;
+		if(_mosquitto_read_byte(&context->core.in_packet, &granted_qos)) return 1;
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_handle_unsuback(mqtt3_context *context)
 {
 	uint16_t mid;
 
-	if(!context || context->in_packet.remaining_length != 2){
-		return 1;
+	if(!context){
+		return MOSQ_ERR_INVAL;
 	}
-	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received UNSUBACK");
-	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
+	if(context->core.in_packet.remaining_length != 2){
+		return MOSQ_ERR_PROTOCOL;
+	}
+	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received UNSUBACK");
+	if(_mosquitto_read_uint16(&context->core.in_packet, &mid)) return 1;
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
