@@ -29,7 +29,10 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <string.h>
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <mqtt3.h>
 #include <mqtt3_protocol.h>
 #include <memory_mosq.h>
@@ -41,12 +44,14 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 	uint8_t protocol_version;
 	uint8_t connect_flags;
 	char *client_id;
-	char *will_message = NULL;
+	char *will_message = NULL, *will_topic = NULL;
+	struct mosquitto_message *will_struct = NULL;
 	uint8_t will, will_retain, will_qos, clean_session;
 	uint8_t username_flag, password_flag;
-	char *username, *password;
+	char *username, *password = NULL;
 	int i;
 	int rc;
+	struct _mosquitto_acl_user *acl_tail;
 	
 	/* Don't accept multiple CONNECT commands. */
 	if(context->core.state != mosq_cs_new){
@@ -59,14 +64,14 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 		return 1;
 	}
 	if(!protocol_name){
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return 3;
 	}
 	if(strcmp(protocol_name, PROTOCOL_NAME)){
 		mqtt3_log_printf(MOSQ_LOG_INFO, "Invalid protocol \"%s\" in CONNECT from %s.",
-				protocol_name, context->address);
+				protocol_name, context->core.address);
 		_mosquitto_free(protocol_name);
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return MOSQ_ERR_PROTOCOL;
 	}
 	_mosquitto_free(protocol_name);
@@ -77,84 +82,60 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 	}
 	if(protocol_version != PROTOCOL_VERSION){
 		mqtt3_log_printf(MOSQ_LOG_INFO, "Invalid protocol version %d in CONNECT from %s.",
-				protocol_version, context->address);
+				protocol_version, context->core.address);
+		_mosquitto_free(protocol_name);
 		mqtt3_raw_connack(context, 1);
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return MOSQ_ERR_PROTOCOL;
 	}
 
 	if(_mosquitto_read_byte(&context->core.in_packet, &connect_flags)){
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return 1;
 	}
 	clean_session = connect_flags & 0x02;
 	will = connect_flags & 0x04;
 	will_qos = (connect_flags & 0x18) >> 3;
 	will_retain = connect_flags & 0x20;
-	username_flag = connect_flags & 0x40;
-	password_flag = connect_flags & 0x80;
+	password_flag = connect_flags & 0x40;
+	username_flag = connect_flags & 0x80;
 
 	if(_mosquitto_read_uint16(&context->core.in_packet, &(context->core.keepalive))){
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return 1;
 	}
 
 	if(_mosquitto_read_string(&context->core.in_packet, &client_id)){
-		mqtt3_socket_close(context);
+		_mosquitto_socket_close(&context->core);
 		return 1;
 	}
 
-	/* Find if this client already has an entry */
-	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && db->contexts[i]->core.id && !strcmp(db->contexts[i]->core.id, client_id)){
-			/* Client does match. */
-			if(db->contexts[i]->core.sock == -1){
-				/* Client is reconnecting after a disconnect */
-				/* FIXME - does anything else need to be done here? */
-			}else{
-				/* Client is already connected, disconnect old version */
-				mqtt3_log_printf(MOSQ_LOG_ERR, "Client %s already connected, closing old connection.", client_id);
-			}
-			db->contexts[i]->clean_session = clean_session;
-			mqtt3_context_cleanup(db, db->contexts[i], false);
-			db->contexts[i]->core.state = mosq_cs_connected;
-			db->contexts[i]->address = _mosquitto_strdup(context->address);
-			db->contexts[i]->core.sock = context->core.sock;
-			context->core.sock = -1;
-			context->core.state = mosq_cs_disconnecting;
-			context = db->contexts[i];
-			break;
+	/* clientid_prefixes check */
+	if(db->config->clientid_prefixes){
+		if(strncmp(db->config->clientid_prefixes, client_id, strlen(db->config->clientid_prefixes))){
+			mqtt3_raw_connack(context, 2);
+			mqtt3_socket_close(context);
+			return MOSQ_ERR_SUCCESS;
 		}
 	}
 
-	context->core.id = client_id;
-	context->clean_session = clean_session;
-
-	if(connect_flags & 0x04){
-		context->core.will = _mosquitto_calloc(1, sizeof(struct mosquitto_message));
-		if(!context->core.will){
-			mqtt3_socket_close(context);
+	if(will){
+		will_struct = _mosquitto_calloc(1, sizeof(struct mosquitto_message));
+		if(!will_struct){
+			_mosquitto_socket_close(&context->core);
 			return MOSQ_ERR_NOMEM;
 		}
-		if(_mosquitto_read_string(&context->core.in_packet, &context->core.will->topic)){
-			mqtt3_socket_close(context);
+		if(_mosquitto_read_string(&context->core.in_packet, &will_topic)){
+			_mosquitto_socket_close(&context->core);
 			return 1;
 		}
 		if(_mosquitto_read_string(&context->core.in_packet, &will_message)){
-			mqtt3_socket_close(context);
+			_mosquitto_socket_close(&context->core);
 			return 1;
 		}
-		if(will_message){
-			context->core.will->payload = (uint8_t *)will_message;
-			context->core.will->payloadlen = strlen(will_message);
-		}else{
-			context->core.will->payload = NULL;
-			context->core.will->payloadlen = 0;
-		}
-		context->core.will->qos = will_qos;
-		context->core.will->retain = will_retain;
 	}
 
+	/* FIXME - sort memory leaks from auth failures. */
 	if(username_flag){
 		rc = _mosquitto_read_string(&context->core.in_packet, &username);
 		if(rc == MOSQ_ERR_SUCCESS){
@@ -167,6 +148,14 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 					password_flag = 0;
 				}
 			}
+			rc = mqtt3_unpwd_check(db, username, password);
+			if(rc == MOSQ_ERR_AUTH){
+				mqtt3_raw_connack(context, 2);
+				mqtt3_socket_close(context);
+				return MOSQ_ERR_SUCCESS;
+			}else if(rc == MOSQ_ERR_INVAL){
+				return MOSQ_ERR_INVAL;
+			}
 		}else if(rc == MOSQ_ERR_NOMEM){
 			return MOSQ_ERR_NOMEM;
 		}else{
@@ -174,7 +163,75 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 			username_flag = 0;
 		}
 	}
-	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received CONNECT from %s as %s", context->address, client_id);
+
+	if(!username_flag && db->config->allow_anonymous == false){
+		mqtt3_raw_connack(context, 2);
+		mqtt3_socket_close(context);
+		return MOSQ_ERR_SUCCESS;
+	}
+
+	/* Find if this client already has an entry. This must be done *after* any security checks. */
+	for(i=0; i<db->context_count; i++){
+		if(db->contexts[i] && db->contexts[i]->core.id && !strcmp(db->contexts[i]->core.id, client_id)){
+			/* Client does match. */
+			if(db->contexts[i]->core.sock == -1){
+				/* Client is reconnecting after a disconnect */
+				/* FIXME - does anything else need to be done here? */
+			}else{
+				/* Client is already connected, disconnect old version */
+				mqtt3_log_printf(MOSQ_LOG_ERR, "Client %s already connected, closing old connection.", client_id);
+			}
+			db->contexts[i]->core.clean_session = clean_session;
+			mqtt3_context_cleanup(db, db->contexts[i], false);
+			db->contexts[i]->core.state = mosq_cs_connected;
+			db->contexts[i]->core.address = _mosquitto_strdup(context->core.address);
+			db->contexts[i]->core.sock = context->core.sock;
+			context->core.sock = -1;
+			context->core.state = mosq_cs_disconnecting;
+			context = db->contexts[i];
+			break;
+		}
+	}
+
+	context->core.id = client_id;
+	context->core.clean_session = clean_session;
+
+	context->core.will = will_struct;
+	if(context->core.will){
+		context->core.will->topic = will_topic;
+		if(will_message){
+			context->core.will->payload = (uint8_t *)will_message;
+			context->core.will->payloadlen = strlen(will_message);
+		}else{
+			context->core.will->payload = NULL;
+			context->core.will->payloadlen = 0;
+		}
+		context->core.will->qos = will_qos;
+		context->core.will->retain = will_retain;
+	}
+
+	/* Associate user with its ACL, assuming we have ACLs loaded. */
+	if(db->acl_list){
+		acl_tail = db->acl_list;
+		while(acl_tail){
+			if(context->core.username){
+				if(!strcmp(context->core.username, acl_tail->username)){
+					context->acl_list = acl_tail;
+					break;
+				}
+			}else{
+				if(acl_tail->username == NULL){
+					context->acl_list = acl_tail;
+					break;
+				}
+			}
+			acl_tail = acl_tail->next;
+		}
+	}else{
+		context->acl_list = NULL;
+	}
+
+	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received CONNECT from %s as %s", context->core.address, client_id);
 
 	context->core.state = mosq_cs_connected;
 	return mqtt3_raw_connack(context, 0);
@@ -183,15 +240,15 @@ int mqtt3_handle_connect(mosquitto_db *db, mqtt3_context *context)
 int mqtt3_handle_disconnect(mqtt3_context *context)
 {
 	if(!context){
-		return 1;
+		return MOSQ_ERR_INVAL;
 	}
 	if(context->core.in_packet.remaining_length != 0){
 		return MOSQ_ERR_PROTOCOL;
 	}
 	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received DISCONNECT from %s", context->core.id);
 	context->core.state = mosq_cs_disconnecting;
-	mqtt3_socket_close(context);
-	return 0;
+	_mosquitto_socket_close(&context->core);
+	return MOSQ_ERR_SUCCESS;
 }
 
 
@@ -204,7 +261,7 @@ int mqtt3_handle_subscribe(mosquitto_db *db, mqtt3_context *context)
 	uint8_t *payload = NULL, *tmp_payload;
 	uint32_t payloadlen = 0;
 
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received SUBSCRIBE from %s", context->core.id);
 	/* FIXME - plenty of potential for memory leaks here */
 
@@ -226,7 +283,7 @@ int mqtt3_handle_subscribe(mosquitto_db *db, mqtt3_context *context)
 			}
 			if(qos > 2){
 				mqtt3_log_printf(MOSQ_LOG_INFO, "Invalid QoS in subscription command from %s, disconnecting.",
-					context->address);
+					context->core.address);
 				_mosquitto_free(sub);
 				if(payload) _mosquitto_free(payload);
 				return 1;
@@ -238,15 +295,26 @@ int mqtt3_handle_subscribe(mosquitto_db *db, mqtt3_context *context)
 			}
 			if(!strlen(sub)){
 				mqtt3_log_printf(MOSQ_LOG_INFO, "Empty subscription string from %s, disconnecting.",
-					context->address);
+					context->core.address);
 				_mosquitto_free(sub);
 				if(payload) _mosquitto_free(payload);
 				return 1;
 			}
 			mqtt3_log_printf(MOSQ_LOG_DEBUG, "\t%s (QoS %d)", sub, qos);
+			/* FIXME - need to deny access to retained messages. */
+#if 0
+			/* Check for topic access */
+			rc2 = mqtt3_acl_check(db, context, sub, MOSQ_ACL_READ);
+			if(rc2 == MOSQ_ERR_SUCCESS){
+				mqtt3_sub_add(context, sub, qos, &db->subs);
+				if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
+			}else if(rc2 != MOSQ_ERR_ACL_DENIED){
+				rc = 1;
+			}
+#else
 			mqtt3_sub_add(context, sub, qos, &db->subs);
-	
 			if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
+#endif
 			_mosquitto_free(sub);
 		}
 
@@ -273,7 +341,7 @@ int mqtt3_handle_unsubscribe(mosquitto_db *db, mqtt3_context *context)
 	uint16_t mid;
 	char *sub;
 
-	if(!context) return 1;
+	if(!context) return MOSQ_ERR_INVAL;
 	mqtt3_log_printf(MOSQ_LOG_DEBUG, "Received UNSUBSCRIBE from %s", context->core.id);
 
 	if(_mosquitto_read_uint16(&context->core.in_packet, &mid)) return 1;
