@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2010 Roger Light <roger@atchoo.org>
+Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,10 @@ struct _sub_token {
 	char *topic;
 };
 
-static int _subs_process(struct _mosquitto_subhier *hier, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
+static int _subs_process(struct _mosquitto_db *db, struct _mosquitto_subhier *hier, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
 {
 	int rc = 0;
+	int rc2;
 	int client_qos, msg_qos;
 	uint16_t mid;
 	struct _mosquitto_subleaf *leaf;
@@ -66,6 +67,14 @@ static int _subs_process(struct _mosquitto_subhier *hier, const char *source_id,
 			leaf = leaf->next;
 			continue;
 		}
+		/* Check for ACL topic access. */
+		rc2 = mqtt3_acl_check(db, leaf->context, topic, MOSQ_ACL_READ);
+		if(rc2 == MOSQ_ERR_ACL_DENIED){
+			leaf = leaf->next;
+			continue;
+		}else if(rc2 != MOSQ_ERR_SUCCESS){
+			rc = 1;
+		}
 		client_qos = leaf->qos;
 
 		if(qos > client_qos){
@@ -81,7 +90,7 @@ static int _subs_process(struct _mosquitto_subhier *hier, const char *source_id,
 		if(mqtt3_db_message_insert(leaf->context, mid, mosq_md_out, msg_qos, false, stored) == 1) rc = 1;
 		leaf = leaf->next;
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 static int _sub_topic_tokenise(const char *subtopic, struct _sub_token **topics)
@@ -94,7 +103,7 @@ static int _sub_topic_tokenise(const char *subtopic, struct _sub_token **topics)
 	assert(topics);
 
 	local_subtopic = _mosquitto_strdup(subtopic);
-	if(!local_subtopic) return 1;
+	if(!local_subtopic) return MOSQ_ERR_NOMEM;
 
 	token = strtok(local_subtopic, "/");
 	while(token){
@@ -116,7 +125,7 @@ static int _sub_topic_tokenise(const char *subtopic, struct _sub_token **topics)
 	
 	_mosquitto_free(local_subtopic);
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 
 cleanup:
 	_mosquitto_free(local_subtopic);
@@ -142,11 +151,17 @@ static int _sub_add(mqtt3_context *context, int qos, struct _mosquitto_subhier *
 			leaf = subhier->subs;
 			last_leaf = NULL;
 			while(leaf){
+				if(!strcmp(leaf->context->core.id, context->core.id)){
+					/* Client making a second subscription to same topic. Only
+					 * need to update QoS. */
+					leaf->qos = qos;
+					return MOSQ_ERR_SUCCESS;
+				}
 				last_leaf = leaf;
 				leaf = leaf->next;
 			}
 			leaf = _mosquitto_malloc(sizeof(struct _mosquitto_subleaf));
-			if(!leaf) return 1;
+			if(!leaf) return MOSQ_ERR_NOMEM;
 			leaf->next = NULL;
 			leaf->context = context;
 			leaf->qos = qos;
@@ -158,7 +173,7 @@ static int _sub_add(mqtt3_context *context, int qos, struct _mosquitto_subhier *
 				leaf->prev = NULL;
 			}
 		}
-		return 0;
+		return MOSQ_ERR_SUCCESS;
 	}
 
 	branch = subhier->children;
@@ -171,13 +186,14 @@ static int _sub_add(mqtt3_context *context, int qos, struct _mosquitto_subhier *
 	}
 	/* Not found */
 	branch = _mosquitto_calloc(1, sizeof(struct _mosquitto_subhier));
-	if(!branch) return 1;
+	if(!branch) return MOSQ_ERR_NOMEM;
+	branch->topic = _mosquitto_strdup(tokens->topic);
+	if(!branch->topic) return MOSQ_ERR_NOMEM;
 	if(!last){
 		subhier->children = branch;
 	}else{
 		last->next = branch;
 	}
-	branch->topic = _mosquitto_strdup(tokens->topic);
 	return _sub_add(context, qos, branch, tokens->next);
 }
 
@@ -199,11 +215,11 @@ static int _sub_remove(mqtt3_context *context, struct _mosquitto_subhier *subhie
 					leaf->next->prev = leaf->prev;
 				}
 				_mosquitto_free(leaf);
-				return 0;
+				return MOSQ_ERR_SUCCESS;
 			}
 			leaf = leaf->next;
 		}
-		return 0;
+		return MOSQ_ERR_SUCCESS;
 	}
 
 	branch = subhier->children;
@@ -219,18 +235,18 @@ static int _sub_remove(mqtt3_context *context, struct _mosquitto_subhier *subhie
 				_mosquitto_free(branch->topic);
 				_mosquitto_free(branch);
 			}
-			return 0;
+			return MOSQ_ERR_SUCCESS;
 		}
 		last = branch;
 		branch = branch->next;
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
-static int _sub_search(struct _mosquitto_subhier *subhier, struct _sub_token *tokens, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
+static int _sub_search(struct _mosquitto_db *db, struct _mosquitto_subhier *subhier, struct _sub_token *tokens, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
 {
 	/* FIXME - need to take into account source_id if the client is a bridge */
-	struct _mosquitto_subhier *branch, *last = NULL;
+	struct _mosquitto_subhier *branch;
 
 	branch = subhier->children;
 	while(branch){
@@ -238,20 +254,19 @@ static int _sub_search(struct _mosquitto_subhier *subhier, struct _sub_token *to
 			/* The topic matches this subscription.
 			 * Doesn't include # wildcards */
 			if(tokens->next){
-				_sub_search(branch, tokens->next, source_id, topic, qos, retain, stored);
+				_sub_search(db, branch, tokens->next, source_id, topic, qos, retain, stored);
 			}else{
-				_subs_process(branch, source_id, topic, qos, retain, stored);
+				_subs_process(db, branch, source_id, topic, qos, retain, stored);
 			}
 		}else if(!strcmp(branch->topic, "#") && !branch->children){
 			/* The topic matches due to a # wildcard - process the
 			 * subscriptions and return. */
-			_subs_process(branch, source_id, topic, qos, retain, stored);
+			_subs_process(db, branch, source_id, topic, qos, retain, stored);
 			break;
 		}
-		last = branch;
 		branch = branch->next;
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_sub_add(mqtt3_context *context, const char *sub, int qos, struct _mosquitto_subhier *root)
@@ -266,15 +281,15 @@ int mqtt3_sub_add(mqtt3_context *context, const char *sub, int qos, struct _mosq
 
 	if(!strncmp(sub, "$SYS/", 5)){
 		tree = 2;
-		if(strlen(sub+5) == 0) return 0;
+		if(strlen(sub+5) == 0) return MOSQ_ERR_SUCCESS;
 		if(_sub_topic_tokenise(sub+5, &tokens)) return 1;
 	}else if(sub[0] == '/'){
 		tree = 1;
-		if(strlen(sub+1) == 0) return 0;
+		if(strlen(sub+1) == 0) return MOSQ_ERR_SUCCESS;
 		if(_sub_topic_tokenise(sub+1, &tokens)) return 1;
 	}else{
 		tree = 0;
-		if(strlen(sub) == 0) return 0;
+		if(strlen(sub) == 0) return MOSQ_ERR_SUCCESS;
 		if(_sub_topic_tokenise(sub, &tokens)) return 1;
 	}
 
@@ -348,7 +363,7 @@ int mqtt3_sub_remove(mqtt3_context *context, const char *sub, struct _mosquitto_
 	return rc;
 }
 
-int mqtt3_sub_search(struct _mosquitto_subhier *root, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
+int mqtt3_sub_search(struct _mosquitto_db *db, struct _mosquitto_subhier *root, const char *source_id, const char *topic, int qos, int retain, struct mosquitto_msg_store *stored)
 {
 	int rc = 0;
 	int tree;
@@ -378,7 +393,7 @@ int mqtt3_sub_search(struct _mosquitto_subhier *root, const char *source_id, con
 				 */
 				_sub_add(NULL, 0, subhier, tokens);
 			}
-			rc = _sub_search(subhier, tokens, source_id, topic, qos, retain, stored);
+			rc = _sub_search(db, subhier, tokens, source_id, topic, qos, retain, stored);
 		}else if(!strcmp(subhier->topic, "/") && tree == 1){
 			if(retain){
 				/* We have a message that needs to be retained, so ensure that the subscription
@@ -386,7 +401,7 @@ int mqtt3_sub_search(struct _mosquitto_subhier *root, const char *source_id, con
 				 */
 				_sub_add(NULL, 0, subhier, tokens);
 			}
-			rc = _sub_search(subhier, tokens, source_id, topic, qos, retain, stored);
+			rc = _sub_search(db, subhier, tokens, source_id, topic, qos, retain, stored);
 		}else if(!strcmp(subhier->topic, "$SYS") && tree == 2){
 			if(retain){
 				/* We have a message that needs to be retained, so ensure that the subscription
@@ -394,7 +409,7 @@ int mqtt3_sub_search(struct _mosquitto_subhier *root, const char *source_id, con
 				 */
 				_sub_add(NULL, 0, subhier, tokens);
 			}
-			rc = _sub_search(subhier, tokens, source_id, topic, qos, retain, stored);
+			rc = _sub_search(db, subhier, tokens, source_id, topic, qos, retain, stored);
 		}
 		subhier = subhier->next;
 	}
@@ -414,7 +429,7 @@ static int _subs_clean_session(mqtt3_context *context, struct _mosquitto_subhier
 	struct _mosquitto_subhier *child, *last = NULL;
 	struct _mosquitto_subleaf *leaf, *next;
 
-	if(!root) return 0;
+	if(!root) return MOSQ_ERR_SUCCESS;
 
 	leaf = root->subs;
 	while(leaf){
@@ -471,7 +486,7 @@ int mqtt3_subs_clean_session(mqtt3_context *context, struct _mosquitto_subhier *
 		child = child->next;
 	}
 
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 void mqtt3_sub_tree_print(struct _mosquitto_subhier *root, int level)
@@ -501,14 +516,19 @@ void mqtt3_sub_tree_print(struct _mosquitto_subhier *root, int level)
 	}
 }
 
-static int _retain_process(struct mosquitto_msg_store *retained, mqtt3_context *context, const char *sub, int sub_qos)
+static int _retain_process(struct _mosquitto_db *db, struct mosquitto_msg_store *retained, mqtt3_context *context, const char *sub, int sub_qos)
 {
 	int rc = 0;
-	char *topic;
 	int qos;
 	uint16_t mid;
 
-	topic = retained->msg.topic;
+	rc = mqtt3_acl_check(db, context, retained->msg.topic, MOSQ_ACL_READ);
+	if(rc == MOSQ_ERR_ACL_DENIED){
+		return MOSQ_ERR_SUCCESS;
+	}else if(rc != MOSQ_ERR_SUCCESS){
+		return rc;
+	}
+
 	qos = retained->msg.qos;
 
 	if(qos > sub_qos) qos = sub_qos;
@@ -517,13 +537,12 @@ static int _retain_process(struct mosquitto_msg_store *retained, mqtt3_context *
 	}else{
 		mid = 0;
 	}
-	if(mqtt3_db_message_insert(context, mid, mosq_md_out, qos, true, retained) == 1) rc = 1;
-	return rc;
+	return mqtt3_db_message_insert(context, mid, mosq_md_out, qos, true, retained);
 }
 
-static int _retain_search(struct _mosquitto_subhier *subhier, struct _sub_token *tokens, mqtt3_context *context, const char *sub, int sub_qos)
+static int _retain_search(struct _mosquitto_db *db, struct _mosquitto_subhier *subhier, struct _sub_token *tokens, mqtt3_context *context, const char *sub, int sub_qos)
 {
-	struct _mosquitto_subhier *branch, *last = NULL;
+	struct _mosquitto_subhier *branch;
 
 	branch = subhier->children;
 	while(branch){
@@ -533,23 +552,22 @@ static int _retain_search(struct _mosquitto_subhier *subhier, struct _sub_token
 		if(strcmp(branch->topic, "+") && strcmp(branch->topic, "#")){
 			if(!strcmp(tokens->topic, "#") && !tokens->next){
 				if(branch->retained){
-					_retain_process(branch->retained, context, sub, sub_qos);
+					_retain_process(db, branch->retained, context, sub, sub_qos);
 				}
-				_retain_search(branch, tokens, context, sub, sub_qos);
+				_retain_search(db, branch, tokens, context, sub, sub_qos);
 			}else if(!strcmp(branch->topic, tokens->topic) || !strcmp(branch->topic, "+")){
 				if(tokens->next){
-					_retain_search(branch, tokens->next, context, sub, sub_qos);
+					_retain_search(db, branch, tokens->next, context, sub, sub_qos);
 				}else{
 					if(branch->retained){
-						_retain_process(branch->retained, context, sub, sub_qos);
+						_retain_process(db, branch->retained, context, sub, sub_qos);
 					}
 				}
 			}
 		}
-		last = branch;
 		branch = branch->next;
 	}
-	return 0;
+	return MOSQ_ERR_SUCCESS;
 }
 
 int mqtt3_retain_queue(mosquitto_db *db, mqtt3_context *context, const char *sub, int sub_qos)
@@ -577,13 +595,13 @@ int mqtt3_retain_queue(mosquitto_db *db, mqtt3_context *context, const char *sub
 	subhier = db->subs.children;
 	while(subhier){
 		if(!strcmp(subhier->topic, "") && tree == 0){
-			rc = _retain_search(subhier, tokens, context, sub, sub_qos);
+			rc = _retain_search(db, subhier, tokens, context, sub, sub_qos);
 			break;
 		}else if(!strcmp(subhier->topic, "/") && tree == 1){
-			rc = _retain_search(subhier, tokens, context, sub, sub_qos);
+			rc = _retain_search(db, subhier, tokens, context, sub, sub_qos);
 			break;
 		}else if(!strcmp(subhier->topic, "$SYS") && tree == 2){
-			rc = _retain_search(subhier, tokens, context, sub, sub_qos);
+			rc = _retain_search(db, subhier, tokens, context, sub, sub_qos);
 			break;
 		}
 		subhier = subhier->next;
