@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2011 Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -27,11 +27,15 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
+#ifndef CMAKE
 #include <config.h>
+#endif
+
 #include <stdint.h>
 #include <string.h>
 
 #include <mqtt3.h>
+#include <mqtt3_protocol.h>
 
 /* Convert mqtt command (as defined in mqtt3.h) to corresponding string. */
 const char *mqtt3_command_to_string(uint8_t command)
@@ -71,11 +75,11 @@ const char *mqtt3_command_to_string(uint8_t command)
 
 void mqtt3_check_keepalive(mqtt3_context *context)
 {
-	if(context && context->sock != -1 && time(NULL) - context->last_msg_out >= context->keepalive){
-		if(context->connected){
+	if(context && context->core.sock != -1 && time(NULL) - context->core.last_msg_out >= context->core.keepalive){
+		if(context->core.state == mosq_cs_connected){
 			mqtt3_raw_pingreq(context);
 		}else{
-			mqtt3_socket_close(context);
+			_mosquitto_socket_close(&context->core);
 		}
 	}
 }
